@@ -19,9 +19,12 @@
 #' @param trendline.formula Specify formula used to create trendlines. See \code{\link{geom_smooth}}.
 #' @param trendline.alpha Numeric value to set transparency of trendlines in output plots. Should be in the range 0-1.
 #' @param trendline.darken Numeric value to make the trendlines darker color shades of their corresponding scatterplot points. Should be in the range 1-100.
-#' @param density.plot Logical, if \code{TRUE}, then density plots will be added to the output plots. Set to \code{FALSE} to hide density plots. See \code{\link{geom_density}}.
-#' @param scale.x.log Vector describing if output plots should use a log scale on the x-axis. If length of vector == 1, then the value will be used for all output plots. Vector values should be either \code{TRUE} or \code{FALSE}. See \code{\link{scale_x_log10}}.
-#' @param scale.y.log Vector describing if output plots should use a log scale on the y-axis. If length of vector == 1, then the value will be used for all output plots. Vector values should be either \code{TRUE} or \code{FALSE}. See \code{\link{scale_y_log10}}.
+#' @param density.plot Logical, if \code{TRUE}, then density plots will be added to the output plots. Set to \code{FALSE} to hide density plots.
+#' @param density.plot.type String, type of plot geometry to use for density plots: \code{"density"} for \code{\link{geom_density}} or \code{"boxplot"} for \code{\link{geom_boxplot}}. Outliers are hidden from the boxplots.
+#' @param scale.x.log Vector describing if output plots should use a log scale on the x-axis. A pseudo-log scale will be used if any zero or negative values are present. If length of vector == 1, then the value will be used for all output plots. Vector values should be either \code{TRUE} or \code{FALSE}. See \code{\link{scale_x_log10}}.
+#' @param scale.y.log Vector describing if output plots should use a log scale on the y-axis. A pseudo-log scale will be used if any zero or negative values are present. If length of vector == 1, then the value will be used for all output plots. Vector values should be either \code{TRUE} or \code{FALSE}. See \code{\link{scale_y_log10}}.
+#' @param xsigma Numeric, scaling factor for the linear part of psuedo-long transformation of x axis. Used if \code{scale.x.log} is \code{TRUE} and zero or negative values are present. See \code{\link{pseudo_log_trans}}.
+#' @param ysigma Numeric, scaling factor for the linear part of psuedo-long transformation of y axis. Used if \code{scale.y.log} is \code{TRUE} and zero or negative values are present. See \code{\link{pseudo_log_trans}}.
 #' @param xlimits Vector containing minimum and maximum values for the x-axis of the output plots. See \code{\link{scale_x_continuous}}.
 #' @param xbreaks Vector containing the break values used for the x-axis of the output plots. See \code{\link{scale_x_continuous}}.
 #' @param xlabels Vector containing the labels for each break value used for the x-axis of the output plots. See \code{\link{scale_x_continuous}}.
@@ -34,7 +37,9 @@
 #' @param nrow Integer, number of rows to use in the output arranged plot. See \code{\link{ggarrange}}.
 #' @param align Specify how output plots should be arranged. See \code{\link{ggarrange}}.
 #' @param common.legend Specify if arranged plot should use a common legend. See \code{\link{ggarrange}}.
-#' @param legend.position Specify position of common legend for arranged plot. See \code{\link{ggarrange}}.
+#' @param legend.position Specify position of common legend for arranged plot. See \code{\link{ggarrange}}. Use \code{"none"} to hide legend.
+#' @param group.legend.title String, title for plot legend when generating plots with \code{groups}.
+#' @param common.y.axis Logical, if \code{TRUE}, then only one y-axis label and marginal density plot will be provided. If \code{FALSE}, then separate y-axis labels and marginal density plots will be included for each subplot.
 #' @param summary.table Logical, if \code{TRUE}, then a table providing summary statistics will be included at the bottom of the output plot.
 #' @param filename String, filename used to save plot. File extension must be specified. See \code{\link{ggsave}}.
 #' @param width Numeric, specify width of output plot. See \code{\link{ggsave}}.
@@ -84,24 +89,27 @@
 #' )
 #' }
 #' 
-#' @importFrom dplyr group_by sym left_join n rename select summarize
-#' @importFrom ggplot2 aes aes_string coord_flip element_text geom_density geom_point geom_smooth ggplot ggsave guide_legend guides scale_color_manual scale_fill_discrete scale_fill_manual scale_x_continuous
+#' @importFrom dplyr group_by sym left_join n rename select summarize n_distinct
+#' @importFrom ggplot2 aes coord_flip element_text geom_density geom_boxplot geom_point geom_smooth ggplot ggsave guide_legend guides scale_color_manual scale_fill_discrete scale_fill_manual scale_x_continuous
 #' scale_y_continuous theme theme_void unit waiver xlab ylab scale_x_log10 scale_y_log10
 #' @importFrom ggpubr colnames_style get_legend ggarrange ggtexttable tab_add_title tbody_style ttheme
 #' @importFrom grDevices colorRampPalette hcl
 #' @importFrom patchwork plot_layout plot_spacer
+#' @importFrom purrr possibly
 #' @importFrom stats median
 #' @importFrom rlang .data
+#' @importFrom scales pseudo_log_trans
 #' @export
 
 PlotPerformanceByAttribute <- function(subass, subass.column = 2, groups = NULL, attributes, join.type = c("join", "cbind"), groups.color.pal = NULL, drop = TRUE, alpha = 0.4,
-                                       trendline = TRUE, trendline.method = "lm", trendline.formula = NULL, trendline.alpha = 0.5, trendline.darken = 15, density.plot = FALSE,
-                                       scale.x.log = FALSE, scale.y.log = FALSE, xlimits = c(NA, NA), ylimits = c(NA, NA), xbreaks = waiver(), ybreaks = waiver(), xlabels = waiver(), ylabels = waiver(),
-                                       xlab = NULL, ylab = NULL, ncol = NULL, nrow = NULL, align = "hv", common.legend = TRUE, legend.position = "bottom", summary.table = FALSE,
+                                       trendline = TRUE, trendline.method = "lm", trendline.formula = NULL, trendline.alpha = 0.5, trendline.darken = 15, density.plot = FALSE, density.plot.type = c("density", "boxplot"),
+                                       scale.x.log = FALSE, scale.y.log = FALSE, xsigma = 1, ysigma = 1, xlimits = c(NA, NA), ylimits = c(NA, NA), xbreaks = waiver(), ybreaks = waiver(), xlabels = waiver(), ylabels = waiver(),
+                                       xlab = NULL, ylab = NULL, ncol = NULL, nrow = NULL, align = "hv", common.legend = TRUE, legend.position = "bottom", group.legend.title = "Group", common.y.axis = FALSE, summary.table = FALSE,
                                        filename = NULL, width = NA, height = NA, units = c("in", "cm", "mm", "px"), dpi = 300) {
 
-  # Check join type
+  # Check join type and density plot type
   join.type <- match.arg(join.type)
+  density.plot.type <- match.arg(density.plot.type)
 
   # Check trendline.darken value
   if (trendline.darken < 1) {
@@ -109,9 +117,19 @@ PlotPerformanceByAttribute <- function(subass, subass.column = 2, groups = NULL,
   } else if (trendline.darken > 100) {
     warning("trendline.darken set must be in range 1-100. Setting to 100")
   }
+  
+  # Convert group IDs to string if numeric
+  if(is.numeric(groups[[2]])){
+    groups[[2]] <- as.character(groups[[2]])
+  }
 
   # Create dataframe to store plot data
-  plotdata <- subass
+  if (join.type == "join") {
+    plotdata <- subass %>%
+      filter(!is.na(!!sym(colnames(subass)[subass.column]))) # Remove NA values from y-axis plotting column
+  } else{
+    plotdata <- subass
+  }
 
   # Join subass data to groups if they are given
   if (!is.null(groups)) {
@@ -130,6 +148,7 @@ PlotPerformanceByAttribute <- function(subass, subass.column = 2, groups = NULL,
 
   # Create vector to store plots
   plots <- vector("list")
+  plot_legends <- vector("list")
   plotcols <- colnames(attributes)[which(!colnames(attributes) == "SUBID")]
   
   # Check scale.x.log
@@ -145,6 +164,36 @@ PlotPerformanceByAttribute <- function(subass, subass.column = 2, groups = NULL,
   } else if(!length(scale.y.log) == length(plotcols)){
     stop("ERROR: length of scale.y.log does not match number of output plots")
   }
+  
+  # Determine if ncol and nrow should be automatically calculated
+  if (is.null(ncol) & is.null(nrow)) {
+    override <- TRUE
+  } else if(is.null(ncol)){
+    override <- FALSE
+    ncol <- ceiling(length(plotcols)/nrow)
+  } else if(is.null(nrow)){
+    override <- FALSE
+    nrow <- ceiling(length(plotcols)/ncol)
+  } else if ((ncol * nrow) < length(plotcols)) {
+    warning("ncol * nrow is less than the number of generated plots. Overriding ncol and nrow values.", call. = FALSE)
+    override <- TRUE
+  } else {
+    override <- FALSE
+  }
+  
+  # Calculate ncol and nrow automatically if not specified
+  if (override == TRUE) {
+    # For 4 or fewer plots then just use one row
+    if (length(plotcols) <= 4) {
+      nrow <- 1
+      ncol <- length(plotcols)
+      
+      # Otherwise use a square layout
+    } else {
+      ncol <- ceiling(length(plotcols)^0.5)
+      nrow <- ceiling(length(plotcols)^0.5)
+    }
+  }
 
   # Generate plots
   for (col in plotcols) {
@@ -152,7 +201,7 @@ PlotPerformanceByAttribute <- function(subass, subass.column = 2, groups = NULL,
     # Create plot
     if (!is.null(groups)) {
       plot <- ggplot(data = plotdata, aes(x = !!sym(col), y = !!sym(colnames(subass)[subass.column]))) +
-        geom_point(aes_string(fill = "Group"), alpha = alpha, shape = 21, color = "transparent")
+        geom_point(aes(fill = .data[["Group"]]), alpha = alpha, shape = 21, color = "transparent")
     } else {
       plot <- ggplot(data = plotdata, aes(x = !!sym(col), y = !!sym(colnames(subass)[subass.column]))) +
         geom_point(alpha = alpha)
@@ -161,9 +210,24 @@ PlotPerformanceByAttribute <- function(subass, subass.column = 2, groups = NULL,
     # Add trendlines
     if (trendline == TRUE) {
       if (!is.null(groups)) {
-        plot <- plot + geom_smooth(aes_string(color = "Group"), method = trendline.method, formula = trendline.formula)
+        plot <- plot + geom_smooth(aes(color = .data[["Group"]]), method = trendline.method, formula = trendline.formula)
+        
+        # Identify which groups have unique values and thus trendlines
+        if(drop == TRUE){
+          trendline_groups <- plotdata %>%
+            group_by(.data[["Group"]]) %>%
+            summarize(unique = n_distinct(!!sym(col))) %>%
+            filter(unique > 1) %>%
+            select(all_of("Group")) %>%
+            unlist() %>%
+            as.numeric()
+        } else{
+          trendline_groups <- 1:length(unique(groups[[2]]))
+        }
+        
       } else {
         plot <- plot + geom_smooth(method = trendline.method, formula = trendline.formula)
+        trendline_groups = 1 # Specify color group for trendline
       }
     }
 
@@ -179,12 +243,21 @@ PlotPerformanceByAttribute <- function(subass, subass.column = 2, groups = NULL,
 
     # Format colors if color palette specified
     if (!is.null(groups.color.pal)) {
+      
+      manual_colors <- groups.color.pal[which(sort(unique(groups[[2]])) %in% unique(plotdata$Group))]
+      
+      if(drop == TRUE){
+        legend_colors <- manual_colors
+      } else{
+        legend_colors <- groups.color.pal
+      }
+      
       plot <- plot +
-        scale_fill_manual(values = groups.color.pal, name = "Group", drop = drop) +
-        scale_color_manual(values = unlist(lapply(groups.color.pal, function(X) {
+        scale_fill_manual(values = legend_colors, name = group.legend.title, drop = drop) +
+        scale_color_manual(values = unlist(lapply(groups.color.pal[trendline_groups], function(X) {
           colorRampPalette(c(X, "black"))(100)[trendline.darken] # Add darker colors for trendlines
-        })), name = "Group", drop = drop) + 
-        guides(color = guide_legend(override.aes = list(color = groups.color.pal))) # Override colors in legend to be the original colors
+        })), name = group.legend.title, drop = drop) + 
+        guides(color = guide_legend(override.aes = list(color = groups.color.pal[trendline_groups]))) # Override colors in legend to be the original colors
 
       # Format colors if no color palette specified
     } else {
@@ -196,27 +269,41 @@ PlotPerformanceByAttribute <- function(subass, subass.column = 2, groups = NULL,
       }
 
       # Get colors for ggplot
-      gg_colors <- gg_color_hue(length(unique(groups[[2]])))
+      gg_colors <- gg_color_hue(length(unique(groups[[2]])))[which(sort(unique(groups[[2]])) %in% unique(plotdata$Group))]
+      
+      if(drop == TRUE){
+        legend_colors <- gg_colors
+      } else{
+        legend_colors <- gg_color_hue(length(unique(groups[[2]])))
+      }
 
       # Adjust colors
       plot <- plot +
-        scale_fill_discrete(name = "Group", drop = drop) + # Assign name to pallette for points
-        scale_color_manual(values = unlist(lapply(gg_colors, function(X) {
+        scale_fill_manual(values = legend_colors, name = group.legend.title, drop = drop) + # Assign name to palette for points
+        scale_color_manual(values = unlist(lapply(gg_color_hue(length(unique(groups[[2]])))[trendline_groups], function(X) {
           colorRampPalette(c(X, "black"))(100)[trendline.darken] # Add darker colors for trendlines
-        })), name = "Group", drop = drop) + 
-        guides(color = guide_legend(override.aes = list(color = gg_colors))) # Override colors in legend to be the original colors
+        })), name = group.legend.title, drop = drop) + 
+        guides(color = guide_legend(override.aes = list(color = gg_color_hue(length(unique(groups[[2]])))[trendline_groups]))) # Override colors in legend to be the original colors
     }
     
     # Scale x axis
     if(scale.x.log[which(plotcols == col)] == TRUE){ # Log scale
-      plot <- plot + scale_x_log10(limits = xlimits, breaks = xbreaks, labels = xlabels)
+      if(any(plotdata[[col]] <= 0)){
+        plot <- plot + scale_x_continuous(limits = xlimits, breaks = xbreaks, labels = xlabels, trans=pseudo_log_trans(base = 10, sigma = xsigma)) # Psuedo-log if 0 or negative values
+      } else{
+        plot <- plot + scale_x_log10(limits = xlimits, breaks = xbreaks, labels = xlabels)
+      }
     } else{ # Normal scale
       plot <- plot + scale_x_continuous(limits = xlimits, breaks = xbreaks, labels = xlabels)
     }
     
     # Scale y axis
     if(scale.y.log[which(plotcols == col)] == TRUE){ # Log scale
-      plot <- plot + scale_y_log10(limits = ylimits, breaks = ybreaks, labels = ylabels)
+      if(any(plotdata[[colnames(subass)[subass.column]]] <= 0)){
+        plot <- plot + scale_y_continuous(limits = ylimits, breaks = ybreaks, labels = ylabels, trans=pseudo_log_trans(base = 10, sigma = ysigma)) # Psuedo-log if 0 or negative values
+      } else{
+        plot <- plot + scale_y_log10(limits = ylimits, breaks = ybreaks, labels = ylabels)
+      }
     } else{ # Normal scale
       plot <- plot + scale_y_continuous(limits = ylimits, breaks = ybreaks, labels = ylabels)
     }
@@ -231,113 +318,212 @@ PlotPerformanceByAttribute <- function(subass, subass.column = 2, groups = NULL,
       
       if(!is.null(groups)){
         if (!is.null(groups.color.pal)) { # If custom colors exist
-          # Create density plot for x-axis
-          densx <- ggplot(plotdata, aes(x = !!sym(col), fill = !!sym("Group"))) +
-            geom_density(size = 0.2, alpha = 0.4) +
-            scale_fill_manual(values = groups.color.pal, name = "Group") +
-            theme_void()+
-            theme(legend.position = "none")
-          
-          # Create density plot for y-a.xis
-          densy <- ggplot(plotdata, aes(x = !!sym(colnames(subass)[subass.column]), fill = !!sym("Group"))) +
-            geom_density(size = 0.2, alpha = 0.4) +
-            scale_fill_manual(values = groups.color.pal, name = "Group") +
-            theme_void()+
-            theme(legend.position = "none") +
-            coord_flip()
+          if(density.plot.type == "density"){
+            # Create density plot for x-axis
+            densx <- ggplot(plotdata, aes(x = !!sym(col), fill = !!sym("Group"))) +
+              geom_density(size = 0.2, alpha = 0.4) +
+              scale_fill_manual(values = manual_colors, name = group.legend.title) +
+              theme_void()+
+              theme(legend.position = "none")
+            
+            # Create density plot for y-a.xis
+            densy <- ggplot(plotdata, aes(x = !!sym(colnames(subass)[subass.column]), fill = !!sym("Group"))) +
+              geom_density(size = 0.2, alpha = 0.4) +
+              scale_fill_manual(values = manual_colors, name = group.legend.title) +
+              theme_void()+
+              theme(legend.position = "none") +
+              coord_flip()
+          } else if(density.plot.type == "boxplot"){
+            # Create density plot for x-axis
+            densx <- ggplot(plotdata, aes(x = !!sym(col), fill = !!sym("Group"))) +
+              geom_boxplot(size = 0.2, alpha = 0.4, outlier.shape = NA) +
+              scale_fill_manual(values = manual_colors, name = group.legend.title) +
+              theme_void()+
+              theme(legend.position = "none")
+            
+            # Create density plot for y-a.xis
+            densy <- ggplot(plotdata, aes(x = !!sym(colnames(subass)[subass.column]), fill = !!sym("Group"))) +
+              geom_boxplot(size = 0.2, alpha = 0.4, outlier.shape = NA) +
+              scale_fill_manual(values = manual_colors, name = group.legend.title) +
+              theme_void()+
+              theme(legend.position = "none") +
+              coord_flip()
+          }
         } else{ # Use default colors
-          # Create density plot for x-axis
-          densx <- ggplot(plotdata, aes(x = !!sym(col), fill = !!sym("Group"))) +
-            geom_density(size = 0.2, alpha = 0.4) +
-            theme_void()+
-            theme(legend.position = "none")
-          
-          # Create density plot for y-axis
-          densy <- ggplot(plotdata, aes(x = !!sym(colnames(subass)[subass.column]), fill = !!sym("Group"))) +
-            geom_density(size = 0.2, alpha = 0.4) +
-            theme_void()+
-            theme(legend.position = "none") +
-            coord_flip()
+          if(density.plot.type == "density"){
+            # Create density plot for x-axis
+            densx <- ggplot(plotdata, aes(x = !!sym(col), fill = !!sym("Group"))) +
+              geom_density(size = 0.2, alpha = 0.4) +
+              scale_fill_manual(values = gg_colors, name = group.legend.title) +
+              theme_void()+
+              theme(legend.position = "none")
+            
+            # Create density plot for y-axis
+            densy <- ggplot(plotdata, aes(x = !!sym(colnames(subass)[subass.column]), fill = !!sym("Group"))) +
+              geom_density(size = 0.2, alpha = 0.4) +
+              scale_fill_manual(values = gg_colors, name = group.legend.title) +
+              theme_void()+
+              theme(legend.position = "none") +
+              coord_flip()
+          } else if(density.plot.type == "boxplot"){
+            # Create density plot for x-axis
+            densx <- ggplot(plotdata, aes(x = !!sym(col), fill = !!sym("Group"))) +
+              geom_boxplot(size = 0.2, alpha = 0.4, outlier.shape = NA) +
+              scale_fill_manual(values = gg_colors, name = group.legend.title) +
+              theme_void()+
+              theme(legend.position = "none")
+            
+            # Create density plot for y-axis
+            densy <- ggplot(plotdata, aes(x = !!sym(colnames(subass)[subass.column]), fill = !!sym("Group"))) +
+              geom_boxplot(size = 0.2, alpha = 0.4, outlier.shape = NA) +
+              scale_fill_manual(values = gg_colors, name = group.legend.title) +
+              theme_void()+
+              theme(legend.position = "none") +
+              coord_flip()
+          }
         }
       } else{
-        # Create density plot for x-axis
-        densx <- ggplot(plotdata, aes(x = !!sym(col))) +
-          geom_density(fill = "#619CFF", size = 0.2, alpha = 1) +
-          theme_void()
-        
-        # Create density plot for y-axis
-        densy <- ggplot(plotdata, aes(x = !!sym(colnames(subass)[subass.column]))) +
-          geom_density(fill = "#619CFF", size = 0.2, alpha = 1) +
-          theme_void() +
-          coord_flip()
+        if(density.plot.type == "density"){
+          # Create density plot for x-axis
+          densx <- ggplot(plotdata, aes(x = !!sym(col))) +
+            geom_density(fill = "#619CFF", size = 0.2, alpha = 1) +
+            theme_void()
+          
+          # Create density plot for y-axis
+          densy <- ggplot(plotdata, aes(x = !!sym(colnames(subass)[subass.column]))) +
+            geom_density(fill = "#619CFF", size = 0.2, alpha = 1) +
+            theme_void() +
+            coord_flip()
+        } else if(density.plot.type == "boxplot"){
+          # Create density plot for x-axis
+          densx <- ggplot(plotdata, aes(x = !!sym(col))) +
+            geom_boxplot(fill = "#619CFF", size = 0.2, alpha = 1, outlier.shape = NA) +
+            theme_void()
+          
+          # Create density plot for y-axis
+          densy <- ggplot(plotdata, aes(x = !!sym(colnames(subass)[subass.column]))) +
+            geom_boxplot(fill = "#619CFF", size = 0.2, alpha = 1, outlier.shape = NA) +
+            theme_void() +
+            coord_flip()
+        }
       }
       
       # Scale x axis
       if(scale.x.log[which(plotcols == col)] == TRUE){ # Log scale
-        densx <- densx + scale_x_log10(limits = xlimits, breaks = xbreaks, labels = xlabels)
+        if(any(plotdata[[col]] <= 0)){
+          densx <- densx + scale_x_continuous(limits = xlimits, breaks = xbreaks, labels = xlabels, trans=pseudo_log_trans(base = 10, sigma = xsigma)) # Psuedo-log if 0 or negative values
+        } else{
+          densx <- densx + scale_x_log10(limits = xlimits, breaks = xbreaks, labels = xlabels)
+        }
       } else{ # Normal scale
         densx <- densx + scale_x_continuous(limits = xlimits, breaks = xbreaks, labels = xlabels)
       }
       
       # Scale y axis
       if(scale.y.log[which(plotcols == col)] == TRUE){ # Log scale
-        densy <- densy + scale_x_log10(limits = ylimits, breaks = ybreaks, labels = ylabels)
+        if(any(plotdata[[colnames(subass)[subass.column]]] <= 0)){
+          densy <- densy + scale_x_continuous(limits = ylimits, breaks = ybreaks, labels = ylabels, trans=pseudo_log_trans(base = 10, sigma = ysigma)) # Psuedo-log if 0 or negative values
+        } else{
+          densy <- densy + scale_x_log10(limits = ylimits, breaks = ybreaks, labels = ylabels)
+        }
       } else{ # Normal scale
         densy <- densy + scale_x_continuous(limits = ylimits, breaks = ybreaks, labels = ylabels)
       }
       
       # Backup legend
-      plot_legend <- plot
+      plot_legends[[col]] <- plot + theme(legend.title = element_text(face = "bold"))
       
       # Remove legend from plot
       plot <- plot + theme(legend.position = "none")
       
-      # Create arranged plot
-      plot <- densx + 
-        plot_spacer() + 
-        plot + 
-        densy + 
-        plot_layout(
-          ncol = 2, 
-          nrow = 2, 
-          widths = c(4, 1),
-          heights = c(1, 4)
-        ) 
+      # Create arranged plot - separate y axes, only one column, or plot is the only plot on a row
+      if(common.y.axis == FALSE | ncol == 1 | (col %in% plotcols[1 + ncol * seq(0,nrow - 1)] & col %in% plotcols[c(ncol * seq(1, nrow), length(plotcols))])){
+        plot <- densx + 
+          plot_spacer() + 
+          plot + 
+          densy + 
+          plot_layout(
+            ncol = 2, 
+            nrow = 2, 
+            widths = c(4, 1),
+            heights = c(1, 4)
+          ) 
+      } else{
+        # First plot on a row
+        if(col %in% plotcols[1 + ncol * seq(0,nrow - 1)]){
+          plot <- densx +
+            plot + 
+            plot_layout(
+              ncol = 1, 
+              nrow = 2,
+              widths = 4,
+              heights = c(1, 4)
+            ) 
+        # Last plot on a row
+        } else if(col %in% plotcols[c(ncol * seq(1, nrow), length(plotcols))]){
+          plot <- plot + ylab("") # Remove y-axis title
+          plot <- densx + 
+            plot_spacer() + 
+            plot + 
+            densy + 
+            plot_layout(
+              ncol = 2, 
+              nrow = 2, 
+              widths = c(4, 1),
+              heights = c(1, 4)
+            ) 
+        # Middle plots
+        } else{
+          plot <- plot + ylab("") # Remove y-axis title
+          plot <- densx + 
+            plot + 
+            plot_layout(
+              ncol = 1, 
+              nrow = 2,
+              widths = 4,
+              heights = c(1, 4)
+            ) 
+        }
+      }
     }
 
     # Store plot in list
     plots[[col]] <- plot
   }
-
-  # Determine if ncol and nrow should be automatically calculated
-  if (is.null(ncol) & is.null(nrow)) {
-    override <- TRUE
-  } else if ((ncol * nrow) < length(plots)) {
-    warning("ncol * nrow is less than the number of generated plots. Overriding ncol and nrow values.", call. = FALSE)
-    override <- TRUE
-  } else {
-    override <- FALSE
-  }
-
-  # Calculate ncol and nrow automatically if not specified
-  if (override == TRUE) {
-    # For 4 or fewer plots then just use one row
-    if (length(plots) <= 4) {
-      nrow <- 1
-      ncol <- length(plots)
-
-      # Otherwise use a square layout
-    } else {
-      ncol <- ceiling(length(plots)^0.5)
-      nrow <- ceiling(length(plots)^0.5)
-    }
+  
+  # Specify arranged plot widths
+  # - Currently, if there are fewer plots on the last row, density.plot == TRUE, and common.y.axis == TRUE, then the width of the last plot in the last row is narrower compared to the others
+  #   A potential fix to this would be to add plot spacers to all of the plots in that column and then adjust the arrange_widths to be 5 for that column instead of 4, but then there will be 
+  #   unequal amounts of white space between the different columns
+  if(density.plot == TRUE & common.y.axis == TRUE){
+    arrange_width = c(rep(4,(ncol-1)), 5) # Need to have wider last plot because it contains the density plot
+  } else{
+    arrange_width = 1
   }
 
   # Arrange plots
   if(density.plot == TRUE){
-    arrangeplot <- ggarrange(plotlist = plots, ncol = ncol, nrow = nrow, align = align, common.legend = common.legend, legend = legend.position, legend.grob = get_legend(plot_legend))
+    
+    # Try grabbing legends until one works - sometimes needed when drop == TRUE
+    pass <-  FALSE # Logical for if legend worked
+    for(i in 1:length(plot_legends)){ # Loop through legends
+      if(pass == FALSE){
+        try_legend <- possibly(~get_legend(plot_legends[i]), otherwise = NA)
+        legend.grob <- try_legend()
+        if(any(!is.na(legend.grob))){
+          pass <- TRUE
+        }
+      }
+    }
+    
+    # Arrange plot and suppress warning about alignment
+    if(legend.position == "none"){
+      arrangeplot <- ggarrange(plotlist = plots, ncol = ncol, nrow = nrow, align = align, widths = arrange_width, common.legend = common.legend, legend = legend.position)
+    } else{
+      arrangeplot <- suppressWarnings(ggarrange(plotlist = plots, ncol = ncol, nrow = nrow, align = align, widths = arrange_width, common.legend = common.legend, legend = legend.position, legend.grob = legend.grob))
+    }
   } else{
-    arrangeplot <- ggarrange(plotlist = plots, ncol = ncol, nrow = nrow, align = align, common.legend = common.legend, legend = legend.position)
+    arrangeplot <- ggarrange(plotlist = plots, ncol = ncol, nrow = nrow, align = align, widths = arrange_width, common.legend = common.legend, legend = legend.position)
   }
 
   # Add summary stats table
@@ -378,7 +564,7 @@ PlotPerformanceByAttribute <- function(subass, subass.column = 2, groups = NULL,
         theme = ttheme(
           colnames.style = colnames_style(color = "Black", fill = "grey"),
           padding = unit(c(2, 2), "mm"),
-          tbody.style = tbody_style(size = 8, color = "black", fill = groups.color.pal)
+          tbody.style = tbody_style(size = 8, color = "black", fill = manual_colors)
         )
       )
     } else { # No color ramp specified
